"""
Pydantic Models for CMDB - router/route_map

Runtime validation models for router/route_map configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class RouteMapRuleMatchOriginEnum(str, Enum):
    """Allowed values for match_origin field in rule."""
    NONE = "none"
    EGP = "egp"
    IGP = "igp"
    INCOMPLETE = "incomplete"

class RouteMapRuleSetOriginEnum(str, Enum):
    """Allowed values for set_origin field in rule."""
    NONE = "none"
    EGP = "egp"
    IGP = "igp"
    INCOMPLETE = "incomplete"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class RouteMapRuleSetExtcommunitySoo(BaseModel):
    """
    Child table model for rule.set-extcommunity-soo.
    
    Site-of-Origin extended community.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    community: str | None = Field(max_length=79, default=None, description="Community (format = AA:NN).")
class RouteMapRuleSetExtcommunityRt(BaseModel):
    """
    Child table model for rule.set-extcommunity-rt.
    
    Route Target extended community.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    community: str | None = Field(max_length=79, default=None, description="AA:NN.")
class RouteMapRuleSetCommunity(BaseModel):
    """
    Child table model for rule.set-community.
    
    BGP community attribute.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    community: str | None = Field(max_length=79, default=None, description="Attribute: AA|AA:NN|internet|local-AS|no-advertise|no-export (exact match required for well known communities).")
class RouteMapRuleSetAspath(BaseModel):
    """
    Child table model for rule.set-aspath.
    
    Prepend BGP AS path attribute.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    as_: str | None = Field(max_length=79, default=None, serialization_alias="as", description="AS number (0 - 4294967295). Use quotes for repeating numbers, For example, \"1 1 2\".")
class RouteMapRule(BaseModel):
    """
    Child table model for rule.
    
    Rule.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Rule ID.")    
    action: Literal["permit", "deny"] | None = Field(default="permit", description="Action.")    
    match_as_path: str | None = Field(max_length=35, default=None, description="Match BGP AS path list.")  # datasource: ['router.aspath-list.name']    
    match_community: str | None = Field(max_length=35, default=None, description="Match BGP community list.")  # datasource: ['router.community-list.name']    
    match_extcommunity: str | None = Field(max_length=35, default=None, description="Match BGP extended community list.")  # datasource: ['router.extcommunity-list.name']    
    match_community_exact: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable exact matching of communities.")    
    match_extcommunity_exact: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable exact matching of extended communities.")    
    match_origin: RouteMapRuleMatchOriginEnum | None = Field(default=RouteMapRuleMatchOriginEnum.NONE, description="Match BGP origin code.")    
    match_interface: str | None = Field(max_length=15, default=None, description="Match interface configuration.")  # datasource: ['system.interface.name']    
    match_ip_address: str | None = Field(max_length=35, default=None, description="Match IP address permitted by access-list or prefix-list.")  # datasource: ['router.access-list.name', 'router.prefix-list.name']    
    match_ip6_address: str | None = Field(max_length=35, default=None, description="Match IPv6 address permitted by access-list6 or prefix-list6.")  # datasource: ['router.access-list6.name', 'router.prefix-list6.name']    
    match_ip_nexthop: str | None = Field(max_length=35, default=None, description="Match next hop IP address passed by access-list or prefix-list.")  # datasource: ['router.access-list.name', 'router.prefix-list.name']    
    match_ip6_nexthop: str | None = Field(max_length=35, default=None, description="Match next hop IPv6 address passed by access-list6 or prefix-list6.")  # datasource: ['router.access-list6.name', 'router.prefix-list6.name']    
    match_metric: int | None = Field(ge=0, le=4294967295, default=None, description="Match metric for redistribute routes.")    
    match_route_type: Literal["external-type1", "external-type2", "none"] | None = Field(default=None, description="Match route type.")    
    match_tag: int | None = Field(ge=0, le=4294967295, default=None, description="Match tag.")    
    match_vrf: int | None = Field(ge=0, le=511, default=None, description="Match VRF ID.")    
    match_suppress: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable matching of suppressed original neighbor.")    
    set_aggregator_as: int | None = Field(ge=0, le=4294967295, default=0, description="BGP aggregator AS.")    
    set_aggregator_ip: str = Field(default="0.0.0.0", description="BGP aggregator IP.")    
    set_aspath_action: Literal["prepend", "replace"] | None = Field(default="prepend", description="Specify preferred action of set-aspath.")    
    set_aspath: list[RouteMapRuleSetAspath] = Field(default_factory=list, description="Prepend BGP AS path attribute.")    
    set_atomic_aggregate: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable BGP atomic aggregate attribute.")    
    set_community_delete: str | None = Field(max_length=35, default=None, description="Delete communities matching community list.")  # datasource: ['router.community-list.name']    
    set_community: list[RouteMapRuleSetCommunity] = Field(default_factory=list, description="BGP community attribute.")    
    set_community_additive: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable adding set-community to existing community.")    
    set_dampening_reachability_half_life: int | None = Field(ge=0, le=45, default=0, description="Reachability half-life time for the penalty (1 - 45 min, 0 = unset).")    
    set_dampening_reuse: int | None = Field(ge=0, le=20000, default=0, description="Value to start reusing a route (1 - 20000, 0 = unset).")    
    set_dampening_suppress: int | None = Field(ge=0, le=20000, default=0, description="Value to start suppressing a route (1 - 20000, 0 = unset).")    
    set_dampening_max_suppress: int | None = Field(ge=0, le=255, default=0, description="Maximum duration to suppress a route (1 - 255 min, 0 = unset).")    
    set_dampening_unreachability_half_life: int | None = Field(ge=0, le=45, default=0, description="Unreachability Half-life time for the penalty (1 - 45 min, 0 = unset).")    
    set_extcommunity_rt: list[RouteMapRuleSetExtcommunityRt] = Field(default_factory=list, description="Route Target extended community.")    
    set_extcommunity_soo: list[RouteMapRuleSetExtcommunitySoo] = Field(default_factory=list, description="Site-of-Origin extended community.")    
    set_ip_nexthop: str | None = Field(default=None, description="IP address of next hop.")    
    set_ip_prefsrc: str | None = Field(default=None, description="IP address of preferred source.")    
    set_vpnv4_nexthop: str | None = Field(default=None, description="IP address of VPNv4 next-hop.")    
    set_ip6_nexthop: str | None = Field(default=None, description="IPv6 global address of next hop.")    
    set_ip6_nexthop_local: str | None = Field(default=None, description="IPv6 local address of next hop.")    
    set_vpnv6_nexthop: str | None = Field(default=None, description="IPv6 global address of VPNv6 next-hop.")    
    set_vpnv6_nexthop_local: str | None = Field(default=None, description="IPv6 link-local address of VPNv6 next-hop.")    
    set_local_preference: int | None = Field(ge=0, le=4294967295, default=None, description="BGP local preference path attribute.")    
    set_metric: int | None = Field(ge=0, le=4294967295, default=None, description="Metric value.")    
    set_metric_type: Literal["external-type1", "external-type2", "none"] | None = Field(default=None, description="Metric type.")    
    set_originator_id: str | None = Field(default=None, description="BGP originator ID attribute.")    
    set_origin: RouteMapRuleSetOriginEnum | None = Field(default=RouteMapRuleSetOriginEnum.NONE, description="BGP origin code.")    
    set_tag: int | None = Field(ge=0, le=4294967295, default=None, description="Tag value.")    
    set_weight: int | None = Field(ge=0, le=4294967295, default=None, description="BGP weight for routing table.")    
    set_route_tag: int | None = Field(ge=0, le=4294967295, default=None, description="Route tag for routing table.")    
    set_priority: int | None = Field(ge=1, le=65535, default=None, description="Priority for routing table.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class RouteMapModel(BaseModel):
    """
    Pydantic model for router/route_map configuration.
    
    Configure route maps.
    
    Validation Rules:        - name: max_length=35 pattern=        - comments: max_length=127 pattern=        - rule: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=35, description="Name.")    
    comments: str | None = Field(max_length=127, default=None, description="Optional comments.")    
    rule: list[RouteMapRule] = Field(default_factory=list, description="Rule.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "RouteMapModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_rule_references(self, client: Any) -> list[str]:
        """
        Validate rule references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/community-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = RouteMapModel(
            ...     rule=[{"set-community-delete": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_rule_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.route_map.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "rule", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("set-community-delete")
            else:
                value = getattr(item, "set-community-delete", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.router.community_list.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Rule '{value}' not found in "
                    "router/community-list"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_rule_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "RouteMapModel",    "RouteMapRule",    "RouteMapRule.SetAspath",    "RouteMapRule.SetCommunity",    "RouteMapRule.SetExtcommunityRt",    "RouteMapRule.SetExtcommunitySoo",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:14.286718Z
# ============================================================================


"""Validation helpers for system/ntp - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "key",  # Key for authentication.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "ntpsync": "disable",
    "type": "fortiguard",
    "syncinterval": 60,
    "source-ip": "0.0.0.0",
    "source-ip6": "::",
    "server-mode": "disable",
    "authentication": "disable",
    "key-type": "MD5",
    "key-id": 0,
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "ntpsync": "option",  # Enable/disable setting the FortiGate system time by synchron
    "type": "option",  # Use the FortiGuard NTP server or any other available NTP Ser
    "syncinterval": "integer",  # NTP synchronization interval (1 - 1440 min).
    "ntpserver": "string",  # Configure the FortiGate to connect to any available third-pa
    "source-ip": "ipv4-address",  # Source IP address for communication to the NTP server.
    "source-ip6": "ipv6-address",  # Source IPv6 address for communication to the NTP server.
    "server-mode": "option",  # Enable/disable FortiGate NTP Server Mode. Your FortiGate bec
    "authentication": "option",  # Enable/disable authentication.
    "key-type": "option",  # Key type for authentication (MD5, SHA1, SHA256).
    "key": "password",  # Key for authentication.
    "key-id": "integer",  # Key ID for authentication.
    "interface": "string",  # FortiGate interface(s) with NTP server mode enabled. Devices
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "ntpsync": "Enable/disable setting the FortiGate system time by synchronizing with an NTP Server.",
    "type": "Use the FortiGuard NTP server or any other available NTP Server.",
    "syncinterval": "NTP synchronization interval (1 - 1440 min).",
    "ntpserver": "Configure the FortiGate to connect to any available third-party NTP server.",
    "source-ip": "Source IP address for communication to the NTP server.",
    "source-ip6": "Source IPv6 address for communication to the NTP server.",
    "server-mode": "Enable/disable FortiGate NTP Server Mode. Your FortiGate becomes an NTP server for other devices on your network. The FortiGate relays NTP requests to its configured NTP server.",
    "authentication": "Enable/disable authentication.",
    "key-type": "Key type for authentication (MD5, SHA1, SHA256).",
    "key": "Key for authentication.",
    "key-id": "Key ID for authentication.",
    "interface": "FortiGate interface(s) with NTP server mode enabled. Devices on your network can contact these interfaces for NTP services.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "syncinterval": {"type": "integer", "min": 1, "max": 1440},
    "key-id": {"type": "integer", "min": 0, "max": 4294967295},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "ntpserver": {
        "id": {
            "type": "integer",
            "help": "NTP server ID.",
            "required": True,
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "server": {
            "type": "string",
            "help": "IP address or hostname of the NTP Server.",
            "required": True,
            "default": "",
            "max_length": 63,
        },
        "ntpv3": {
            "type": "option",
            "help": "Enable to use NTPv3 instead of NTPv4.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "authentication": {
            "type": "option",
            "help": "Enable/disable authentication.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "key-type": {
            "type": "option",
            "help": "Select NTP authentication type.",
            "default": "MD5",
            "options": ["MD5", "SHA1", "SHA256"],
        },
        "key": {
            "type": "password",
            "help": "Key for MD5(NTPv3)/SHA1(NTPv4)/SHA256(NTPv4) authentication.",
            "required": True,
            "max_length": 64,
        },
        "key-id": {
            "type": "integer",
            "help": "Key ID for authentication.",
            "required": True,
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "ip-type": {
            "type": "option",
            "help": "Choose to connect to IPv4 or/and IPv6 NTP server.",
            "default": "Both",
            "options": ["IPv6", "IPv4", "Both"],
        },
        "interface-select-method": {
            "type": "option",
            "help": "Specify how to select outgoing interface to reach server.",
            "default": "auto",
            "options": ["auto", "sdwan", "specify"],
        },
        "interface": {
            "type": "string",
            "help": "Specify outgoing interface to reach server.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "vrf-select": {
            "type": "integer",
            "help": "VRF ID used for connection to server.",
            "default": 0,
            "min_value": 0,
            "max_value": 511,
        },
    },
    "interface": {
        "interface-name": {
            "type": "string",
            "help": "Interface name.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_NTPSYNC = [
    "enable",
    "disable",
]
VALID_BODY_TYPE = [
    "fortiguard",
    "custom",
]
VALID_BODY_SERVER_MODE = [
    "enable",
    "disable",
]
VALID_BODY_AUTHENTICATION = [
    "enable",
    "disable",
]
VALID_BODY_KEY_TYPE = [
    "MD5",
    "SHA1",
    "SHA256",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_system_ntp_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for system/ntp."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_system_ntp_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new system/ntp object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "ntpsync" in payload:
        is_valid, error = _validate_enum_field(
            "ntpsync",
            payload["ntpsync"],
            VALID_BODY_NTPSYNC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-mode" in payload:
        is_valid, error = _validate_enum_field(
            "server-mode",
            payload["server-mode"],
            VALID_BODY_SERVER_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "authentication" in payload:
        is_valid, error = _validate_enum_field(
            "authentication",
            payload["authentication"],
            VALID_BODY_AUTHENTICATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "key-type" in payload:
        is_valid, error = _validate_enum_field(
            "key-type",
            payload["key-type"],
            VALID_BODY_KEY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_system_ntp_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update system/ntp."""
    # Validate enum values using central function
    if "ntpsync" in payload:
        is_valid, error = _validate_enum_field(
            "ntpsync",
            payload["ntpsync"],
            VALID_BODY_NTPSYNC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-mode" in payload:
        is_valid, error = _validate_enum_field(
            "server-mode",
            payload["server-mode"],
            VALID_BODY_SERVER_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "authentication" in payload:
        is_valid, error = _validate_enum_field(
            "authentication",
            payload["authentication"],
            VALID_BODY_AUTHENTICATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "key-type" in payload:
        is_valid, error = _validate_enum_field(
            "key-type",
            payload["key-type"],
            VALID_BODY_KEY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "system/ntp",
    "category": "cmdb",
    "api_path": "system/ntp",
    "help": "Configure system NTP information.",
    "total_fields": 12,
    "required_fields_count": 1,
    "fields_with_defaults_count": 9,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""FortiOS CMDB - RegionImage category"""

from ..region_image_base import RegionImage as RegionImageBase
from .upload import Upload

__all__ = [
    "RegionImage",
    "Upload",
]


class RegionImage(RegionImageBase):
    """RegionImage endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """RegionImage endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.upload = Upload(client)



"""
Pydantic Models for CMDB - system/dhcp6/server

Runtime validation models for system/dhcp6/server configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class ServerOptionsTypeEnum(str, Enum):
    """Allowed values for type_ field in options."""
    HEX = "hex"
    STRING = "string"
    IP6 = "ip6"
    FQDN = "fqdn"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ServerPrefixRange(BaseModel):
    """
    Child table model for prefix-range.
    
    DHCP prefix configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    start_prefix: str = Field(default="::", description="Start of prefix range.")    
    end_prefix: str = Field(default="::", description="End of prefix range.")    
    prefix_length: int = Field(ge=1, le=128, default=0, description="Prefix length.")
class ServerOptionsVciString(BaseModel):
    """
    Child table model for options.vci-string.
    
    One or more VCI strings in quotes separated by spaces.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    vci_string: str = Field(max_length=255, description="VCI strings.")
class ServerOptions(BaseModel):
    """
    Child table model for options.
    
    DHCPv6 options.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    code: int = Field(ge=0, le=255, default=0, description="DHCPv6 option code.")    
    type_: ServerOptionsTypeEnum | None = Field(default=ServerOptionsTypeEnum.HEX, serialization_alias="type", description="DHCPv6 option type.")    
    value: str | None = Field(max_length=312, default=None, description="DHCPv6 option value (hexadecimal value must be even).")    
    ip6: list[str] = Field(default_factory=list, description="DHCP option IP6s.")    
    vci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable vendor class option matching. When enabled only DHCP requests with a matching VCI are served with this option.")    
    vci_string: list[ServerOptionsVciString] = Field(default_factory=list, description="One or more VCI strings in quotes separated by spaces.")
class ServerIpRangeVciString(BaseModel):
    """
    Child table model for ip-range.vci-string.
    
    One or more VCI strings in quotes separated by spaces.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    vci_string: str = Field(max_length=255, description="VCI strings.")
class ServerIpRange(BaseModel):
    """
    Child table model for ip-range.
    
    DHCP IP range configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    start_ip: str = Field(default="::", description="Start of IP range.")    
    end_ip: str = Field(default="::", description="End of IP range.")    
    vci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable vendor class option matching. When enabled only DHCP requests with a matching VC are served with this range.")    
    vci_string: list[ServerIpRangeVciString] = Field(default_factory=list, description="One or more VCI strings in quotes separated by spaces.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class ServerModel(BaseModel):
    """
    Pydantic model for system/dhcp6/server configuration.
    
    Configure DHCPv6 servers.
    
    Validation Rules:        - id_: min=0 max=4294967295 pattern=        - status: pattern=        - rapid_commit: pattern=        - lease_time: min=300 max=8640000 pattern=        - dns_service: pattern=        - dns_search_list: pattern=        - dns_server1: pattern=        - dns_server2: pattern=        - dns_server3: pattern=        - dns_server4: pattern=        - domain: max_length=35 pattern=        - subnet: pattern=        - interface: max_length=15 pattern=        - delegated_prefix_route: pattern=        - options: pattern=        - upstream_interface: max_length=15 pattern=        - delegated_prefix_iaid: min=0 max=4294967295 pattern=        - ip_mode: pattern=        - prefix_mode: pattern=        - prefix_range: pattern=        - ip_range: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable this DHCPv6 configuration.")    
    rapid_commit: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable allow/disallow rapid commit.")    
    lease_time: int | None = Field(ge=300, le=8640000, default=604800, description="Lease time in seconds, 0 means unlimited.")    
    dns_service: Literal["delegated", "default", "specify"] | None = Field(default="specify", description="Options for assigning DNS servers to DHCPv6 clients.")    
    dns_search_list: Literal["delegated", "specify"] | None = Field(default="specify", description="DNS search list options.")    
    dns_server1: str | None = Field(default="::", description="DNS server 1.")    
    dns_server2: str | None = Field(default="::", description="DNS server 2.")    
    dns_server3: str | None = Field(default="::", description="DNS server 3.")    
    dns_server4: str | None = Field(default="::", description="DNS server 4.")    
    domain: str | None = Field(max_length=35, default=None, description="Domain name suffix for the IP addresses that the DHCP server assigns to clients.")    
    subnet: str = Field(default="::/0", description="Subnet or subnet-id if the IP mode is delegated.")    
    interface: str = Field(max_length=15, description="DHCP server can assign IP configurations to clients connected to this interface.")  # datasource: ['system.interface.name']    
    delegated_prefix_route: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable automatically adding of routing for delegated prefix.")    
    options: list[ServerOptions] = Field(default_factory=list, description="DHCPv6 options.")    
    upstream_interface: str = Field(max_length=15, description="Interface name from where delegated information is provided.")  # datasource: ['system.interface.name']    
    delegated_prefix_iaid: int = Field(ge=0, le=4294967295, default=0, description="IAID of obtained delegated-prefix from the upstream interface.")    
    ip_mode: Literal["range", "delegated"] | None = Field(default="range", description="Method used to assign client IP.")    
    prefix_mode: Literal["dhcp6", "ra"] | None = Field(default="dhcp6", description="Assigning a prefix from a DHCPv6 client or RA.")    
    prefix_range: list[ServerPrefixRange] = Field(default_factory=list, description="DHCP prefix configuration.")    
    ip_range: list[ServerIpRange] = Field(default_factory=list, description="DHCP IP range configuration.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('upstream_interface')
    @classmethod
    def validate_upstream_interface(cls, v: Any) -> Any:
        """
        Validate upstream_interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ServerModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ServerModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dhcp6.server.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_upstream_interface_references(self, client: Any) -> list[str]:
        """
        Validate upstream_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ServerModel(
            ...     upstream_interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_upstream_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dhcp6.server.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "upstream_interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Upstream-Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_upstream_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ServerModel",    "ServerOptions",    "ServerOptions.VciString",    "ServerPrefixRange",    "ServerIpRange",    "ServerIpRange.VciString",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.950584Z
# ============================================================================


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_MODE: Literal["standalone", "a-a", "a-p"]
VALID_BODY_SYNC_PACKET_BALANCE: Literal["enable", "disable"]
VALID_BODY_UNICAST_HB: Literal["enable", "disable"]
VALID_BODY_LOAD_BALANCE_ALL: Literal["enable", "disable"]
VALID_BODY_SYNC_CONFIG: Literal["enable", "disable"]
VALID_BODY_ENCRYPTION: Literal["enable", "disable"]
VALID_BODY_AUTHENTICATION: Literal["enable", "disable"]
VALID_BODY_HB_INTERVAL_IN_MILLISECONDS: Literal["100ms", "10ms"]
VALID_BODY_GRATUITOUS_ARPS: Literal["enable", "disable"]
VALID_BODY_SESSION_PICKUP: Literal["enable", "disable"]
VALID_BODY_SESSION_PICKUP_CONNECTIONLESS: Literal["enable", "disable"]
VALID_BODY_SESSION_PICKUP_EXPECTATION: Literal["enable", "disable"]
VALID_BODY_SESSION_PICKUP_NAT: Literal["enable", "disable"]
VALID_BODY_SESSION_PICKUP_DELAY: Literal["enable", "disable"]
VALID_BODY_LINK_FAILED_SIGNAL: Literal["enable", "disable"]
VALID_BODY_UPGRADE_MODE: Literal["simultaneous", "uninterruptible", "local-only", "secondary-only"]
VALID_BODY_STANDALONE_MGMT_VDOM: Literal["enable", "disable"]
VALID_BODY_HA_MGMT_STATUS: Literal["enable", "disable"]
VALID_BODY_STANDALONE_CONFIG_SYNC: Literal["enable", "disable"]
VALID_BODY_UNICAST_STATUS: Literal["enable", "disable"]
VALID_BODY_SCHEDULE: Literal["none", "leastconnection", "round-robin", "weight-round-robin", "random", "ip", "ipport"]
VALID_BODY_OVERRIDE: Literal["enable", "disable"]
VALID_BODY_PINGSERVER_SECONDARY_FORCE_RESET: Literal["enable", "disable"]
VALID_BODY_VCLUSTER_STATUS: Literal["enable", "disable"]
VALID_BODY_HA_DIRECT: Literal["enable", "disable"]
VALID_BODY_SSD_FAILOVER: Literal["enable", "disable"]
VALID_BODY_MEMORY_COMPATIBLE_MODE: Literal["enable", "disable"]
VALID_BODY_MEMORY_BASED_FAILOVER: Literal["enable", "disable"]
VALID_BODY_CHECK_SECONDARY_DEV_HEALTH: Literal["enable", "disable"]
VALID_BODY_IPSEC_PHASE2_PROPOSAL: Literal["aes128-sha1", "aes128-sha256", "aes128-sha384", "aes128-sha512", "aes192-sha1", "aes192-sha256", "aes192-sha384", "aes192-sha512", "aes256-sha1", "aes256-sha256", "aes256-sha384", "aes256-sha512", "aes128gcm", "aes256gcm", "chacha20poly1305"]
VALID_BODY_BOUNCE_INTF_UPON_FAILOVER: Literal["enable", "disable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_MODE",
    "VALID_BODY_SYNC_PACKET_BALANCE",
    "VALID_BODY_UNICAST_HB",
    "VALID_BODY_LOAD_BALANCE_ALL",
    "VALID_BODY_SYNC_CONFIG",
    "VALID_BODY_ENCRYPTION",
    "VALID_BODY_AUTHENTICATION",
    "VALID_BODY_HB_INTERVAL_IN_MILLISECONDS",
    "VALID_BODY_GRATUITOUS_ARPS",
    "VALID_BODY_SESSION_PICKUP",
    "VALID_BODY_SESSION_PICKUP_CONNECTIONLESS",
    "VALID_BODY_SESSION_PICKUP_EXPECTATION",
    "VALID_BODY_SESSION_PICKUP_NAT",
    "VALID_BODY_SESSION_PICKUP_DELAY",
    "VALID_BODY_LINK_FAILED_SIGNAL",
    "VALID_BODY_UPGRADE_MODE",
    "VALID_BODY_STANDALONE_MGMT_VDOM",
    "VALID_BODY_HA_MGMT_STATUS",
    "VALID_BODY_STANDALONE_CONFIG_SYNC",
    "VALID_BODY_UNICAST_STATUS",
    "VALID_BODY_SCHEDULE",
    "VALID_BODY_OVERRIDE",
    "VALID_BODY_PINGSERVER_SECONDARY_FORCE_RESET",
    "VALID_BODY_VCLUSTER_STATUS",
    "VALID_BODY_HA_DIRECT",
    "VALID_BODY_SSD_FAILOVER",
    "VALID_BODY_MEMORY_COMPATIBLE_MODE",
    "VALID_BODY_MEMORY_BASED_FAILOVER",
    "VALID_BODY_CHECK_SECONDARY_DEV_HEALTH",
    "VALID_BODY_IPSEC_PHASE2_PROPOSAL",
    "VALID_BODY_BOUNCE_INTF_UPON_FAILOVER",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""
Data conversion and cleaning helpers for FortiOS API.

Provides utilities for:
- Type conversion (bool to enable/disable)
- Data filtering and cleaning
- URL encoding for path parameters
"""

from typing import Any
from urllib.parse import quote


def convert_boolean_to_str(value: bool | str | int | None) -> str | None:
    """
    Convert Python boolean to FortiOS enable/disable string.

    FortiOS API typically uses 'enable'/'disable' instead of true/false.

    Args:
        value: Boolean, string, or None

    Returns:
        'enable', 'disable', the original string, or None
    """
    if value is None:
        return None
    if isinstance(value, bool):
        return "enable" if value else "disable"
    return str(value)


def filter_empty_values(payload: dict[str, Any]) -> dict[str, Any]:
    """
    Remove None values and empty collections from payload.

    Useful for cleaning up payloads before sending to FortiOS API,
    which may reject empty lists or None values in certain contexts.

    Args:
        payload: Dictionary to clean

    Returns:
        Dictionary with None and empty values removed
    """
    cleaned: dict[str, Any] = {}

    for key, value in payload.items():
        # Skip None values
        if value is None:
            continue

        # Skip empty lists and dicts
        if isinstance(value, (list, dict)) and not value:
            continue

        cleaned[key] = value

    return cleaned


def quote_path_param(value: str | int) -> str:
    """
    URL-encode a path parameter for safe inclusion in API endpoint URLs.

    FortiOS object names (mkey values) can contain characters like '/' that
    conflict with URL path separators. This function encodes such characters
    to their percent-encoded equivalents.

    Args:
        value: The path parameter value (typically an mkey like name, policyid, etc.)

    Returns:
        URL-encoded string safe for use in URL paths

    Examples:
        >>> quote_path_param("CGNAT_100.64.0.0/10")
        'CGNAT_100.64.0.0%2F10'
        >>> quote_path_param("simple-name")
        'simple-name'
        >>> quote_path_param(123)
        '123'
    """
    # Convert to string and encode, making NO characters safe
    # (even '/' must be encoded since it's a path separator)
    return quote(str(value), safe="")



"""Validation helpers for firewall/vip - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "server-type",  # Protocol to be load balanced by the virtual server (also called the server load balance virtual IP).
    "extip",  # IP address or address range on the external interface that you want to map to an address or address range on the destination network.
    "mappedip",  # IP address or address range on the destination network to which the external IP address is mapped.
    "extintf",  # Interface connected to the source network that receives the packets that will be forwarded to the destination network.
    "extport",  # Incoming port number range that you want to map to a port number range on the destination network.
    "ssl-certificate",  # Name of the certificate to use for SSL handshake.
    "ipv6-mappedip",  # Range of mapped IPv6 addresses. Specify the start IPv6 address followed by a space and the end IPv6 address.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "id": 0,
    "uuid": "00000000-0000-0000-0000-000000000000",
    "type": "static-nat",
    "server-type": "",
    "dns-mapping-ttl": 0,
    "ldb-method": "static",
    "src-vip-filter": "disable",
    "extip": "",
    "h2-support": "enable",
    "h3-support": "disable",
    "nat44": "enable",
    "nat46": "disable",
    "add-nat46-route": "enable",
    "mapped-addr": "",
    "extintf": "",
    "arp-reply": "enable",
    "http-redirect": "disable",
    "persistence": "none",
    "nat-source-vip": "disable",
    "portforward": "disable",
    "status": "enable",
    "protocol": "tcp",
    "extport": "",
    "mappedport": "",
    "gratuitous-arp-interval": 0,
    "portmapping-type": "1-to-1",
    "empty-cert-action": "block",
    "user-agent-detect": "enable",
    "client-cert": "enable",
    "http-cookie-domain-from-host": "disable",
    "http-cookie-domain": "",
    "http-cookie-path": "",
    "http-cookie-generation": 0,
    "http-cookie-age": 60,
    "http-cookie-share": "same-ip",
    "https-cookie-secure": "disable",
    "http-multiplex": "disable",
    "http-multiplex-ttl": 15,
    "http-multiplex-max-request": 0,
    "http-multiplex-max-concurrent-request": 0,
    "http-ip-header": "disable",
    "http-ip-header-name": "",
    "outlook-web-access": "disable",
    "weblogic-server": "disable",
    "websphere-server": "disable",
    "ssl-mode": "half",
    "ssl-dh-bits": "2048",
    "ssl-algorithm": "high",
    "ssl-server-algorithm": "client",
    "ssl-pfs": "require",
    "ssl-min-version": "tls-1.1",
    "ssl-max-version": "tls-1.3",
    "ssl-server-min-version": "client",
    "ssl-server-max-version": "client",
    "ssl-accept-ffdhe-groups": "enable",
    "ssl-send-empty-frags": "enable",
    "ssl-client-fallback": "enable",
    "ssl-client-renegotiation": "secure",
    "ssl-client-session-state-type": "both",
    "ssl-client-session-state-timeout": 30,
    "ssl-client-session-state-max": 1000,
    "ssl-client-rekey-count": 0,
    "ssl-server-renegotiation": "enable",
    "ssl-server-session-state-type": "both",
    "ssl-server-session-state-timeout": 60,
    "ssl-server-session-state-max": 100,
    "ssl-http-location-conversion": "disable",
    "ssl-http-match-host": "enable",
    "ssl-hpkp": "disable",
    "ssl-hpkp-primary": "",
    "ssl-hpkp-backup": "",
    "ssl-hpkp-age": 5184000,
    "ssl-hpkp-include-subdomains": "disable",
    "ssl-hsts": "disable",
    "ssl-hsts-age": 5184000,
    "ssl-hsts-include-subdomains": "disable",
    "max-embryonic-connections": 1000,
    "color": 0,
    "ipv6-mappedip": "",
    "ipv6-mappedport": "",
    "one-click-gslb-server": "disable",
    "gslb-hostname": "",
    "gslb-domain-name": "",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Virtual IP name.
    "id": "integer",  # Custom defined ID.
    "uuid": "uuid",  # Universally Unique Identifier (UUID; automatically assigned 
    "comment": "var-string",  # Comment.
    "type": "option",  # Configure a static NAT, load balance, server load balance, a
    "server-type": "option",  # Protocol to be load balanced by the virtual server (also cal
    "dns-mapping-ttl": "integer",  # DNS mapping TTL (Set to zero to use TTL in DNS response, def
    "ldb-method": "option",  # Method used to distribute sessions to real servers.
    "src-filter": "string",  # Source address filter. Each address must be either an IP/sub
    "src-vip-filter": "option",  # Enable/disable use of 'src-filter' to match destinations for
    "service": "string",  # Service name.
    "extip": "user",  # IP address or address range on the external interface that y
    "extaddr": "string",  # External FQDN address name.
    "h2-support": "option",  # Enable/disable HTTP2 support (default = enable).
    "h3-support": "option",  # Enable/disable HTTP3/QUIC support (default = disable).
    "quic": "string",  # QUIC setting.
    "nat44": "option",  # Enable/disable NAT44.
    "nat46": "option",  # Enable/disable NAT46.
    "add-nat46-route": "option",  # Enable/disable adding NAT46 route.
    "mappedip": "string",  # IP address or address range on the destination network to wh
    "mapped-addr": "string",  # Mapped FQDN address name.
    "extintf": "string",  # Interface connected to the source network that receives the 
    "arp-reply": "option",  # Enable to respond to ARP requests for this virtual IP addres
    "http-redirect": "option",  # Enable/disable redirection of HTTP to HTTPS.
    "persistence": "option",  # Configure how to make sure that clients connect to the same 
    "nat-source-vip": "option",  # Enable/disable forcing the source NAT mapped IP to the exter
    "portforward": "option",  # Enable/disable port forwarding.
    "status": "option",  # Enable/disable VIP.
    "protocol": "option",  # Protocol to use when forwarding packets.
    "extport": "user",  # Incoming port number range that you want to map to a port nu
    "mappedport": "user",  # Port number range on the destination network to which the ex
    "gratuitous-arp-interval": "integer",  # Enable to have the VIP send gratuitous ARPs. 0=disabled. Set
    "srcintf-filter": "string",  # Interfaces to which the VIP applies. Separate the names with
    "portmapping-type": "option",  # Port mapping type.
    "empty-cert-action": "option",  # Action for an empty client certificate.
    "user-agent-detect": "option",  # Enable/disable detecting device type by HTTP user-agent if n
    "client-cert": "option",  # Enable/disable requesting client certificate.
    "realservers": "string",  # Select the real servers that this server load balancing VIP 
    "http-cookie-domain-from-host": "option",  # Enable/disable use of HTTP cookie domain from host field in 
    "http-cookie-domain": "string",  # Domain that HTTP cookie persistence should apply to.
    "http-cookie-path": "string",  # Limit HTTP cookie persistence to the specified path.
    "http-cookie-generation": "integer",  # Generation of HTTP cookie to be accepted. Changing invalidat
    "http-cookie-age": "integer",  # Time in minutes that client web browsers should keep a cooki
    "http-cookie-share": "option",  # Control sharing of cookies across virtual servers. Use of sa
    "https-cookie-secure": "option",  # Enable/disable verification that inserted HTTPS cookies are 
    "http-multiplex": "option",  # Enable/disable HTTP multiplexing.
    "http-multiplex-ttl": "integer",  # Time-to-live for idle connections to servers.
    "http-multiplex-max-request": "integer",  # Maximum number of requests that a multiplex server can handl
    "http-multiplex-max-concurrent-request": "integer",  # Maximum number of concurrent requests that a multiplex serve
    "http-ip-header": "option",  # For HTTP multiplexing, enable to add the original client IP 
    "http-ip-header-name": "string",  # For HTTP multiplexing, enter a custom HTTPS header name. The
    "outlook-web-access": "option",  # Enable to add the Front-End-Https header for Microsoft Outlo
    "weblogic-server": "option",  # Enable to add an HTTP header to indicate SSL offloading for 
    "websphere-server": "option",  # Enable to add an HTTP header to indicate SSL offloading for 
    "ssl-mode": "option",  # Apply SSL offloading between the client and the FortiGate (h
    "ssl-certificate": "string",  # Name of the certificate to use for SSL handshake.
    "ssl-dh-bits": "option",  # Number of bits to use in the Diffie-Hellman exchange for RSA
    "ssl-algorithm": "option",  # Permitted encryption algorithms for SSL sessions according t
    "ssl-cipher-suites": "string",  # SSL/TLS cipher suites acceptable from a client, ordered by p
    "ssl-server-algorithm": "option",  # Permitted encryption algorithms for the server side of SSL f
    "ssl-server-cipher-suites": "string",  # SSL/TLS cipher suites to offer to a server, ordered by prior
    "ssl-pfs": "option",  # Select the cipher suites that can be used for SSL perfect fo
    "ssl-min-version": "option",  # Lowest SSL/TLS version acceptable from a client.
    "ssl-max-version": "option",  # Highest SSL/TLS version acceptable from a client.
    "ssl-server-min-version": "option",  # Lowest SSL/TLS version acceptable from a server. Use the cli
    "ssl-server-max-version": "option",  # Highest SSL/TLS version acceptable from a server. Use the cl
    "ssl-accept-ffdhe-groups": "option",  # Enable/disable FFDHE cipher suite for SSL key exchange.
    "ssl-send-empty-frags": "option",  # Enable/disable sending empty fragments to avoid CBC IV attac
    "ssl-client-fallback": "option",  # Enable/disable support for preventing Downgrade Attacks on c
    "ssl-client-renegotiation": "option",  # Allow, deny, or require secure renegotiation of client sessi
    "ssl-client-session-state-type": "option",  # How to expire SSL sessions for the segment of the SSL connec
    "ssl-client-session-state-timeout": "integer",  # Number of minutes to keep client to FortiGate SSL session st
    "ssl-client-session-state-max": "integer",  # Maximum number of client to FortiGate SSL session states to 
    "ssl-client-rekey-count": "integer",  # Maximum length of data in MB before triggering a client reke
    "ssl-server-renegotiation": "option",  # Enable/disable secure renegotiation to comply with RFC 5746.
    "ssl-server-session-state-type": "option",  # How to expire SSL sessions for the segment of the SSL connec
    "ssl-server-session-state-timeout": "integer",  # Number of minutes to keep FortiGate to Server SSL session st
    "ssl-server-session-state-max": "integer",  # Maximum number of FortiGate to Server SSL session states to 
    "ssl-http-location-conversion": "option",  # Enable to replace HTTP with HTTPS in the reply's Location HT
    "ssl-http-match-host": "option",  # Enable/disable HTTP host matching for location conversion.
    "ssl-hpkp": "option",  # Enable/disable including HPKP header in response.
    "ssl-hpkp-primary": "string",  # Certificate to generate primary HPKP pin from.
    "ssl-hpkp-backup": "string",  # Certificate to generate backup HPKP pin from.
    "ssl-hpkp-age": "integer",  # Number of seconds the client should honor the HPKP setting.
    "ssl-hpkp-report-uri": "var-string",  # URL to report HPKP violations to.
    "ssl-hpkp-include-subdomains": "option",  # Indicate that HPKP header applies to all subdomains.
    "ssl-hsts": "option",  # Enable/disable including HSTS header in response.
    "ssl-hsts-age": "integer",  # Number of seconds the client should honor the HSTS setting.
    "ssl-hsts-include-subdomains": "option",  # Indicate that HSTS header applies to all subdomains.
    "monitor": "string",  # Name of the health check monitor to use when polling to dete
    "max-embryonic-connections": "integer",  # Maximum number of incomplete connections.
    "color": "integer",  # Color of icon on the GUI.
    "ipv6-mappedip": "user",  # Range of mapped IPv6 addresses. Specify the start IPv6 addre
    "ipv6-mappedport": "user",  # IPv6 port number range on the destination network to which t
    "one-click-gslb-server": "option",  # Enable/disable one click GSLB server integration with FortiG
    "gslb-hostname": "string",  # Hostname to use within the configured FortiGSLB domain.
    "gslb-domain-name": "string",  # Domain to use when integrating with FortiGSLB.
    "gslb-public-ips": "string",  # Publicly accessible IP addresses for the FortiGSLB service.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Virtual IP name.",
    "id": "Custom defined ID.",
    "uuid": "Universally Unique Identifier (UUID; automatically assigned but can be manually reset).",
    "comment": "Comment.",
    "type": "Configure a static NAT, load balance, server load balance, access proxy, DNS translation, or FQDN VIP.",
    "server-type": "Protocol to be load balanced by the virtual server (also called the server load balance virtual IP).",
    "dns-mapping-ttl": "DNS mapping TTL (Set to zero to use TTL in DNS response, default = 0).",
    "ldb-method": "Method used to distribute sessions to real servers.",
    "src-filter": "Source address filter. Each address must be either an IP/subnet (x.x.x.x/n) or a range (x.x.x.x-y.y.y.y). Separate addresses with spaces.",
    "src-vip-filter": "Enable/disable use of 'src-filter' to match destinations for the reverse SNAT rule.",
    "service": "Service name.",
    "extip": "IP address or address range on the external interface that you want to map to an address or address range on the destination network.",
    "extaddr": "External FQDN address name.",
    "h2-support": "Enable/disable HTTP2 support (default = enable).",
    "h3-support": "Enable/disable HTTP3/QUIC support (default = disable).",
    "quic": "QUIC setting.",
    "nat44": "Enable/disable NAT44.",
    "nat46": "Enable/disable NAT46.",
    "add-nat46-route": "Enable/disable adding NAT46 route.",
    "mappedip": "IP address or address range on the destination network to which the external IP address is mapped.",
    "mapped-addr": "Mapped FQDN address name.",
    "extintf": "Interface connected to the source network that receives the packets that will be forwarded to the destination network.",
    "arp-reply": "Enable to respond to ARP requests for this virtual IP address. Enabled by default.",
    "http-redirect": "Enable/disable redirection of HTTP to HTTPS.",
    "persistence": "Configure how to make sure that clients connect to the same server every time they make a request that is part of the same session.",
    "nat-source-vip": "Enable/disable forcing the source NAT mapped IP to the external IP for all traffic.",
    "portforward": "Enable/disable port forwarding.",
    "status": "Enable/disable VIP.",
    "protocol": "Protocol to use when forwarding packets.",
    "extport": "Incoming port number range that you want to map to a port number range on the destination network.",
    "mappedport": "Port number range on the destination network to which the external port number range is mapped.",
    "gratuitous-arp-interval": "Enable to have the VIP send gratuitous ARPs. 0=disabled. Set from 5 up to 8640000 seconds to enable.",
    "srcintf-filter": "Interfaces to which the VIP applies. Separate the names with spaces.",
    "portmapping-type": "Port mapping type.",
    "empty-cert-action": "Action for an empty client certificate.",
    "user-agent-detect": "Enable/disable detecting device type by HTTP user-agent if no client certificate is provided.",
    "client-cert": "Enable/disable requesting client certificate.",
    "realservers": "Select the real servers that this server load balancing VIP will distribute traffic to.",
    "http-cookie-domain-from-host": "Enable/disable use of HTTP cookie domain from host field in HTTP.",
    "http-cookie-domain": "Domain that HTTP cookie persistence should apply to.",
    "http-cookie-path": "Limit HTTP cookie persistence to the specified path.",
    "http-cookie-generation": "Generation of HTTP cookie to be accepted. Changing invalidates all existing cookies.",
    "http-cookie-age": "Time in minutes that client web browsers should keep a cookie. Default is 60 minutes. 0 = no time limit.",
    "http-cookie-share": "Control sharing of cookies across virtual servers. Use of same-ip means a cookie from one virtual server can be used by another. Disable stops cookie sharing.",
    "https-cookie-secure": "Enable/disable verification that inserted HTTPS cookies are secure.",
    "http-multiplex": "Enable/disable HTTP multiplexing.",
    "http-multiplex-ttl": "Time-to-live for idle connections to servers.",
    "http-multiplex-max-request": "Maximum number of requests that a multiplex server can handle before disconnecting sessions (default = unlimited).",
    "http-multiplex-max-concurrent-request": "Maximum number of concurrent requests that a multiplex server can handle (default = unlimited).",
    "http-ip-header": "For HTTP multiplexing, enable to add the original client IP address in the X-Forwarded-For HTTP header.",
    "http-ip-header-name": "For HTTP multiplexing, enter a custom HTTPS header name. The original client IP address is added to this header. If empty, X-Forwarded-For is used.",
    "outlook-web-access": "Enable to add the Front-End-Https header for Microsoft Outlook Web Access.",
    "weblogic-server": "Enable to add an HTTP header to indicate SSL offloading for a WebLogic server.",
    "websphere-server": "Enable to add an HTTP header to indicate SSL offloading for a WebSphere server.",
    "ssl-mode": "Apply SSL offloading between the client and the FortiGate (half) or from the client to the FortiGate and from the FortiGate to the server (full).",
    "ssl-certificate": "Name of the certificate to use for SSL handshake.",
    "ssl-dh-bits": "Number of bits to use in the Diffie-Hellman exchange for RSA encryption of SSL sessions.",
    "ssl-algorithm": "Permitted encryption algorithms for SSL sessions according to encryption strength.",
    "ssl-cipher-suites": "SSL/TLS cipher suites acceptable from a client, ordered by priority.",
    "ssl-server-algorithm": "Permitted encryption algorithms for the server side of SSL full mode sessions according to encryption strength.",
    "ssl-server-cipher-suites": "SSL/TLS cipher suites to offer to a server, ordered by priority.",
    "ssl-pfs": "Select the cipher suites that can be used for SSL perfect forward secrecy (PFS). Applies to both client and server sessions.",
    "ssl-min-version": "Lowest SSL/TLS version acceptable from a client.",
    "ssl-max-version": "Highest SSL/TLS version acceptable from a client.",
    "ssl-server-min-version": "Lowest SSL/TLS version acceptable from a server. Use the client setting by default.",
    "ssl-server-max-version": "Highest SSL/TLS version acceptable from a server. Use the client setting by default.",
    "ssl-accept-ffdhe-groups": "Enable/disable FFDHE cipher suite for SSL key exchange.",
    "ssl-send-empty-frags": "Enable/disable sending empty fragments to avoid CBC IV attacks (SSL 3.0 & TLS 1.0 only). May need to be disabled for compatibility with older systems.",
    "ssl-client-fallback": "Enable/disable support for preventing Downgrade Attacks on client connections (RFC 7507).",
    "ssl-client-renegotiation": "Allow, deny, or require secure renegotiation of client sessions to comply with RFC 5746.",
    "ssl-client-session-state-type": "How to expire SSL sessions for the segment of the SSL connection between the client and the FortiGate.",
    "ssl-client-session-state-timeout": "Number of minutes to keep client to FortiGate SSL session state.",
    "ssl-client-session-state-max": "Maximum number of client to FortiGate SSL session states to keep.",
    "ssl-client-rekey-count": "Maximum length of data in MB before triggering a client rekey (0 = disable).",
    "ssl-server-renegotiation": "Enable/disable secure renegotiation to comply with RFC 5746.",
    "ssl-server-session-state-type": "How to expire SSL sessions for the segment of the SSL connection between the server and the FortiGate.",
    "ssl-server-session-state-timeout": "Number of minutes to keep FortiGate to Server SSL session state.",
    "ssl-server-session-state-max": "Maximum number of FortiGate to Server SSL session states to keep.",
    "ssl-http-location-conversion": "Enable to replace HTTP with HTTPS in the reply's Location HTTP header field.",
    "ssl-http-match-host": "Enable/disable HTTP host matching for location conversion.",
    "ssl-hpkp": "Enable/disable including HPKP header in response.",
    "ssl-hpkp-primary": "Certificate to generate primary HPKP pin from.",
    "ssl-hpkp-backup": "Certificate to generate backup HPKP pin from.",
    "ssl-hpkp-age": "Number of seconds the client should honor the HPKP setting.",
    "ssl-hpkp-report-uri": "URL to report HPKP violations to.",
    "ssl-hpkp-include-subdomains": "Indicate that HPKP header applies to all subdomains.",
    "ssl-hsts": "Enable/disable including HSTS header in response.",
    "ssl-hsts-age": "Number of seconds the client should honor the HSTS setting.",
    "ssl-hsts-include-subdomains": "Indicate that HSTS header applies to all subdomains.",
    "monitor": "Name of the health check monitor to use when polling to determine a virtual server's connectivity status.",
    "max-embryonic-connections": "Maximum number of incomplete connections.",
    "color": "Color of icon on the GUI.",
    "ipv6-mappedip": "Range of mapped IPv6 addresses. Specify the start IPv6 address followed by a space and the end IPv6 address.",
    "ipv6-mappedport": "IPv6 port number range on the destination network to which the external port number range is mapped.",
    "one-click-gslb-server": "Enable/disable one click GSLB server integration with FortiGSLB.",
    "gslb-hostname": "Hostname to use within the configured FortiGSLB domain.",
    "gslb-domain-name": "Domain to use when integrating with FortiGSLB.",
    "gslb-public-ips": "Publicly accessible IP addresses for the FortiGSLB service.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 79},
    "id": {"type": "integer", "min": 0, "max": 65535},
    "dns-mapping-ttl": {"type": "integer", "min": 0, "max": 604800},
    "mapped-addr": {"type": "string", "max_length": 79},
    "extintf": {"type": "string", "max_length": 35},
    "gratuitous-arp-interval": {"type": "integer", "min": 5, "max": 8640000},
    "http-cookie-domain": {"type": "string", "max_length": 35},
    "http-cookie-path": {"type": "string", "max_length": 35},
    "http-cookie-generation": {"type": "integer", "min": 0, "max": 4294967295},
    "http-cookie-age": {"type": "integer", "min": 0, "max": 525600},
    "http-multiplex-ttl": {"type": "integer", "min": 0, "max": 2147483647},
    "http-multiplex-max-request": {"type": "integer", "min": 0, "max": 2147483647},
    "http-multiplex-max-concurrent-request": {"type": "integer", "min": 0, "max": 2147483647},
    "http-ip-header-name": {"type": "string", "max_length": 35},
    "ssl-client-session-state-timeout": {"type": "integer", "min": 1, "max": 14400},
    "ssl-client-session-state-max": {"type": "integer", "min": 1, "max": 10000},
    "ssl-client-rekey-count": {"type": "integer", "min": 200, "max": 1048576},
    "ssl-server-session-state-timeout": {"type": "integer", "min": 1, "max": 14400},
    "ssl-server-session-state-max": {"type": "integer", "min": 1, "max": 10000},
    "ssl-hpkp-primary": {"type": "string", "max_length": 79},
    "ssl-hpkp-backup": {"type": "string", "max_length": 79},
    "ssl-hpkp-age": {"type": "integer", "min": 60, "max": 157680000},
    "ssl-hsts-age": {"type": "integer", "min": 60, "max": 157680000},
    "max-embryonic-connections": {"type": "integer", "min": 0, "max": 100000},
    "color": {"type": "integer", "min": 0, "max": 32},
    "gslb-hostname": {"type": "string", "max_length": 35},
    "gslb-domain-name": {"type": "string", "max_length": 255},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "src-filter": {
        "range": {
            "type": "string",
            "help": "Source-filter range.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "service": {
        "name": {
            "type": "string",
            "help": "Service name.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "extaddr": {
        "name": {
            "type": "string",
            "help": "Address name.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "quic": {
        "max-idle-timeout": {
            "type": "integer",
            "help": "Maximum idle timeout milliseconds (1 - 60000, default = 30000).",
            "default": 30000,
            "min_value": 1,
            "max_value": 60000,
        },
        "max-udp-payload-size": {
            "type": "integer",
            "help": "Maximum UDP payload size in bytes (1200 - 1500, default = 1500).",
            "default": 1500,
            "min_value": 1200,
            "max_value": 1500,
        },
        "active-connection-id-limit": {
            "type": "integer",
            "help": "Active connection ID limit (1 - 8, default = 2).",
            "default": 2,
            "min_value": 1,
            "max_value": 8,
        },
        "ack-delay-exponent": {
            "type": "integer",
            "help": "ACK delay exponent (1 - 20, default = 3).",
            "default": 3,
            "min_value": 1,
            "max_value": 20,
        },
        "max-ack-delay": {
            "type": "integer",
            "help": "Maximum ACK delay in milliseconds (1 - 16383, default = 25).",
            "default": 25,
            "min_value": 1,
            "max_value": 16383,
        },
        "max-datagram-frame-size": {
            "type": "integer",
            "help": "Maximum datagram frame size in bytes (1 - 1500, default = 1500).",
            "default": 1500,
            "min_value": 1,
            "max_value": 1500,
        },
        "active-migration": {
            "type": "option",
            "help": "Enable/disable active migration (default = disable).",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "grease-quic-bit": {
            "type": "option",
            "help": "Enable/disable grease QUIC bit (default = enable).",
            "default": "enable",
            "options": ["enable", "disable"],
        },
    },
    "mappedip": {
        "range": {
            "type": "string",
            "help": "Mapped IP range.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "srcintf-filter": {
        "interface-name": {
            "type": "string",
            "help": "Interface name.",
            "default": "",
            "max_length": 79,
        },
    },
    "realservers": {
        "id": {
            "type": "integer",
            "help": "Real server ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "type": {
            "type": "option",
            "help": "Type of address.",
            "required": True,
            "default": "ip",
            "options": ["ip", "address"],
        },
        "address": {
            "type": "string",
            "help": "Dynamic address of the real server.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
        "ip": {
            "type": "user",
            "help": "IP address of the real server.",
            "required": True,
            "default": "",
        },
        "port": {
            "type": "integer",
            "help": "Port for communicating with the real server. Required if port forwarding is enabled.",
            "default": 0,
            "min_value": 1,
            "max_value": 65535,
        },
        "status": {
            "type": "option",
            "help": "Set the status of the real server to active so that it can accept traffic, or on standby or disabled so no traffic is sent.",
            "default": "active",
            "options": ["active", "standby", "disable"],
        },
        "weight": {
            "type": "integer",
            "help": "Weight of the real server. If weighted load balancing is enabled, the server with the highest weight gets more connections.",
            "default": 1,
            "min_value": 1,
            "max_value": 255,
        },
        "holddown-interval": {
            "type": "integer",
            "help": "Time in seconds that the system waits before re-activating a previously down active server in the active-standby mode. This is to prevent any flapping issues.",
            "default": 300,
            "min_value": 30,
            "max_value": 65535,
        },
        "healthcheck": {
            "type": "option",
            "help": "Enable to check the responsiveness of the real server before forwarding traffic.",
            "default": "vip",
            "options": ["disable", "enable", "vip"],
        },
        "http-host": {
            "type": "string",
            "help": "HTTP server domain name in HTTP header.",
            "default": "",
            "max_length": 63,
        },
        "translate-host": {
            "type": "option",
            "help": "Enable/disable translation of hostname/IP from virtual server to real server.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "max-connections": {
            "type": "integer",
            "help": "Max number of active connections that can be directed to the real server. When reached, sessions are sent to other real servers.",
            "default": 0,
            "min_value": 0,
            "max_value": 2147483647,
        },
        "monitor": {
            "type": "string",
            "help": "Name of the health check monitor to use when polling to determine a virtual server's connectivity status.",
        },
        "client-ip": {
            "type": "user",
            "help": "Only clients in this IP range can connect to this real server.",
            "default": "",
        },
        "verify-cert": {
            "type": "option",
            "help": "Enable/disable certificate verification of the real server.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
    },
    "ssl-certificate": {
        "name": {
            "type": "string",
            "help": "Certificate list.",
            "default": "",
            "max_length": 79,
        },
    },
    "ssl-cipher-suites": {
        "priority": {
            "type": "integer",
            "help": "SSL/TLS cipher suites priority.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "cipher": {
            "type": "option",
            "help": "Cipher suite name.",
            "required": True,
            "default": "",
            "options": ["TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-RSA-WITH-DES-CBC-SHA"],
        },
        "versions": {
            "type": "option",
            "help": "SSL/TLS versions that the cipher suite can be used with.",
            "default": "ssl-3.0 tls-1.0 tls-1.1 tls-1.2 tls-1.3",
            "options": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
        },
    },
    "ssl-server-cipher-suites": {
        "priority": {
            "type": "integer",
            "help": "SSL/TLS cipher suites priority.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "cipher": {
            "type": "option",
            "help": "Cipher suite name.",
            "required": True,
            "default": "",
            "options": ["TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-ECDHE-ECDSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-CHACHA20-POLY1305-SHA256", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA", "TLS-DHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-DHE-RSA-WITH-AES-256-CBC-SHA256", "TLS-DHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA", "TLS-DHE-DSS-WITH-AES-128-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-128-GCM-SHA256", "TLS-DHE-DSS-WITH-AES-256-CBC-SHA256", "TLS-DHE-DSS-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA", "TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-AES-128-CBC-SHA", "TLS-RSA-WITH-AES-256-CBC-SHA", "TLS-RSA-WITH-AES-128-CBC-SHA256", "TLS-RSA-WITH-AES-128-GCM-SHA256", "TLS-RSA-WITH-AES-256-CBC-SHA256", "TLS-RSA-WITH-AES-256-GCM-SHA384", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA", "TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-DSS-WITH-CAMELLIA-256-CBC-SHA256", "TLS-DHE-RSA-WITH-SEED-CBC-SHA", "TLS-DHE-DSS-WITH-SEED-CBC-SHA", "TLS-DHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-DHE-DSS-WITH-ARIA-128-CBC-SHA256", "TLS-DHE-DSS-WITH-ARIA-256-CBC-SHA384", "TLS-RSA-WITH-SEED-CBC-SHA", "TLS-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-RSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-ECDSA-WITH-ARIA-128-CBC-SHA256", "TLS-ECDHE-ECDSA-WITH-ARIA-256-CBC-SHA384", "TLS-ECDHE-RSA-WITH-RC4-128-SHA", "TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-DHE-DSS-WITH-3DES-EDE-CBC-SHA", "TLS-RSA-WITH-3DES-EDE-CBC-SHA", "TLS-RSA-WITH-RC4-128-MD5", "TLS-RSA-WITH-RC4-128-SHA", "TLS-DHE-RSA-WITH-DES-CBC-SHA", "TLS-DHE-DSS-WITH-DES-CBC-SHA", "TLS-RSA-WITH-DES-CBC-SHA"],
        },
        "versions": {
            "type": "option",
            "help": "SSL/TLS versions that the cipher suite can be used with.",
            "default": "ssl-3.0 tls-1.0 tls-1.1 tls-1.2 tls-1.3",
            "options": ["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"],
        },
    },
    "monitor": {
        "name": {
            "type": "string",
            "help": "Health monitor name.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "gslb-public-ips": {
        "index": {
            "type": "integer",
            "help": "Index of this public IP setting.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "ip": {
            "type": "ipv4-address-any",
            "help": "The publicly accessible IP address.",
            "default": "0.0.0.0",
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_TYPE = [
    "static-nat",
    "load-balance",
    "server-load-balance",
    "dns-translation",
    "fqdn",
    "access-proxy",
]
VALID_BODY_SERVER_TYPE = [
    "http",
    "https",
    "imaps",
    "pop3s",
    "smtps",
    "ssl",
    "tcp",
    "udp",
    "ip",
]
VALID_BODY_LDB_METHOD = [
    "static",
    "round-robin",
    "weighted",
    "least-session",
    "least-rtt",
    "first-alive",
    "http-host",
]
VALID_BODY_SRC_VIP_FILTER = [
    "disable",
    "enable",
]
VALID_BODY_H2_SUPPORT = [
    "enable",
    "disable",
]
VALID_BODY_H3_SUPPORT = [
    "enable",
    "disable",
]
VALID_BODY_NAT44 = [
    "disable",
    "enable",
]
VALID_BODY_NAT46 = [
    "disable",
    "enable",
]
VALID_BODY_ADD_NAT46_ROUTE = [
    "disable",
    "enable",
]
VALID_BODY_ARP_REPLY = [
    "disable",
    "enable",
]
VALID_BODY_HTTP_REDIRECT = [
    "enable",
    "disable",
]
VALID_BODY_PERSISTENCE = [
    "none",
    "http-cookie",
    "ssl-session-id",
]
VALID_BODY_NAT_SOURCE_VIP = [
    "disable",
    "enable",
]
VALID_BODY_PORTFORWARD = [
    "disable",
    "enable",
]
VALID_BODY_STATUS = [
    "disable",
    "enable",
]
VALID_BODY_PROTOCOL = [
    "tcp",
    "udp",
    "sctp",
    "icmp",
]
VALID_BODY_PORTMAPPING_TYPE = [
    "1-to-1",
    "m-to-n",
]
VALID_BODY_EMPTY_CERT_ACTION = [
    "accept",
    "block",
    "accept-unmanageable",
]
VALID_BODY_USER_AGENT_DETECT = [
    "disable",
    "enable",
]
VALID_BODY_CLIENT_CERT = [
    "disable",
    "enable",
]
VALID_BODY_HTTP_COOKIE_DOMAIN_FROM_HOST = [
    "disable",
    "enable",
]
VALID_BODY_HTTP_COOKIE_SHARE = [
    "disable",
    "same-ip",
]
VALID_BODY_HTTPS_COOKIE_SECURE = [
    "disable",
    "enable",
]
VALID_BODY_HTTP_MULTIPLEX = [
    "enable",
    "disable",
]
VALID_BODY_HTTP_IP_HEADER = [
    "enable",
    "disable",
]
VALID_BODY_OUTLOOK_WEB_ACCESS = [
    "disable",
    "enable",
]
VALID_BODY_WEBLOGIC_SERVER = [
    "disable",
    "enable",
]
VALID_BODY_WEBSPHERE_SERVER = [
    "disable",
    "enable",
]
VALID_BODY_SSL_MODE = [
    "half",
    "full",
]
VALID_BODY_SSL_DH_BITS = [
    "768",
    "1024",
    "1536",
    "2048",
    "3072",
    "4096",
]
VALID_BODY_SSL_ALGORITHM = [
    "high",
    "medium",
    "low",
    "custom",
]
VALID_BODY_SSL_SERVER_ALGORITHM = [
    "high",
    "medium",
    "low",
    "custom",
    "client",
]
VALID_BODY_SSL_PFS = [
    "require",
    "deny",
    "allow",
]
VALID_BODY_SSL_MIN_VERSION = [
    "ssl-3.0",
    "tls-1.0",
    "tls-1.1",
    "tls-1.2",
    "tls-1.3",
]
VALID_BODY_SSL_MAX_VERSION = [
    "ssl-3.0",
    "tls-1.0",
    "tls-1.1",
    "tls-1.2",
    "tls-1.3",
]
VALID_BODY_SSL_SERVER_MIN_VERSION = [
    "ssl-3.0",
    "tls-1.0",
    "tls-1.1",
    "tls-1.2",
    "tls-1.3",
    "client",
]
VALID_BODY_SSL_SERVER_MAX_VERSION = [
    "ssl-3.0",
    "tls-1.0",
    "tls-1.1",
    "tls-1.2",
    "tls-1.3",
    "client",
]
VALID_BODY_SSL_ACCEPT_FFDHE_GROUPS = [
    "enable",
    "disable",
]
VALID_BODY_SSL_SEND_EMPTY_FRAGS = [
    "enable",
    "disable",
]
VALID_BODY_SSL_CLIENT_FALLBACK = [
    "disable",
    "enable",
]
VALID_BODY_SSL_CLIENT_RENEGOTIATION = [
    "allow",
    "deny",
    "secure",
]
VALID_BODY_SSL_CLIENT_SESSION_STATE_TYPE = [
    "disable",
    "time",
    "count",
    "both",
]
VALID_BODY_SSL_SERVER_RENEGOTIATION = [
    "enable",
    "disable",
]
VALID_BODY_SSL_SERVER_SESSION_STATE_TYPE = [
    "disable",
    "time",
    "count",
    "both",
]
VALID_BODY_SSL_HTTP_LOCATION_CONVERSION = [
    "enable",
    "disable",
]
VALID_BODY_SSL_HTTP_MATCH_HOST = [
    "enable",
    "disable",
]
VALID_BODY_SSL_HPKP = [
    "disable",
    "enable",
    "report-only",
]
VALID_BODY_SSL_HPKP_INCLUDE_SUBDOMAINS = [
    "disable",
    "enable",
]
VALID_BODY_SSL_HSTS = [
    "disable",
    "enable",
]
VALID_BODY_SSL_HSTS_INCLUDE_SUBDOMAINS = [
    "disable",
    "enable",
]
VALID_BODY_ONE_CLICK_GSLB_SERVER = [
    "disable",
    "enable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_firewall_vip_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for firewall/vip."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_firewall_vip_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new firewall/vip object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-type" in payload:
        is_valid, error = _validate_enum_field(
            "server-type",
            payload["server-type"],
            VALID_BODY_SERVER_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ldb-method" in payload:
        is_valid, error = _validate_enum_field(
            "ldb-method",
            payload["ldb-method"],
            VALID_BODY_LDB_METHOD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "src-vip-filter" in payload:
        is_valid, error = _validate_enum_field(
            "src-vip-filter",
            payload["src-vip-filter"],
            VALID_BODY_SRC_VIP_FILTER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "h2-support" in payload:
        is_valid, error = _validate_enum_field(
            "h2-support",
            payload["h2-support"],
            VALID_BODY_H2_SUPPORT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "h3-support" in payload:
        is_valid, error = _validate_enum_field(
            "h3-support",
            payload["h3-support"],
            VALID_BODY_H3_SUPPORT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "nat44" in payload:
        is_valid, error = _validate_enum_field(
            "nat44",
            payload["nat44"],
            VALID_BODY_NAT44,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "nat46" in payload:
        is_valid, error = _validate_enum_field(
            "nat46",
            payload["nat46"],
            VALID_BODY_NAT46,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "add-nat46-route" in payload:
        is_valid, error = _validate_enum_field(
            "add-nat46-route",
            payload["add-nat46-route"],
            VALID_BODY_ADD_NAT46_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "arp-reply" in payload:
        is_valid, error = _validate_enum_field(
            "arp-reply",
            payload["arp-reply"],
            VALID_BODY_ARP_REPLY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-redirect" in payload:
        is_valid, error = _validate_enum_field(
            "http-redirect",
            payload["http-redirect"],
            VALID_BODY_HTTP_REDIRECT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "persistence" in payload:
        is_valid, error = _validate_enum_field(
            "persistence",
            payload["persistence"],
            VALID_BODY_PERSISTENCE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "nat-source-vip" in payload:
        is_valid, error = _validate_enum_field(
            "nat-source-vip",
            payload["nat-source-vip"],
            VALID_BODY_NAT_SOURCE_VIP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "portforward" in payload:
        is_valid, error = _validate_enum_field(
            "portforward",
            payload["portforward"],
            VALID_BODY_PORTFORWARD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "protocol" in payload:
        is_valid, error = _validate_enum_field(
            "protocol",
            payload["protocol"],
            VALID_BODY_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "portmapping-type" in payload:
        is_valid, error = _validate_enum_field(
            "portmapping-type",
            payload["portmapping-type"],
            VALID_BODY_PORTMAPPING_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "empty-cert-action" in payload:
        is_valid, error = _validate_enum_field(
            "empty-cert-action",
            payload["empty-cert-action"],
            VALID_BODY_EMPTY_CERT_ACTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "user-agent-detect" in payload:
        is_valid, error = _validate_enum_field(
            "user-agent-detect",
            payload["user-agent-detect"],
            VALID_BODY_USER_AGENT_DETECT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "client-cert" in payload:
        is_valid, error = _validate_enum_field(
            "client-cert",
            payload["client-cert"],
            VALID_BODY_CLIENT_CERT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-cookie-domain-from-host" in payload:
        is_valid, error = _validate_enum_field(
            "http-cookie-domain-from-host",
            payload["http-cookie-domain-from-host"],
            VALID_BODY_HTTP_COOKIE_DOMAIN_FROM_HOST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-cookie-share" in payload:
        is_valid, error = _validate_enum_field(
            "http-cookie-share",
            payload["http-cookie-share"],
            VALID_BODY_HTTP_COOKIE_SHARE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "https-cookie-secure" in payload:
        is_valid, error = _validate_enum_field(
            "https-cookie-secure",
            payload["https-cookie-secure"],
            VALID_BODY_HTTPS_COOKIE_SECURE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-multiplex" in payload:
        is_valid, error = _validate_enum_field(
            "http-multiplex",
            payload["http-multiplex"],
            VALID_BODY_HTTP_MULTIPLEX,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-ip-header" in payload:
        is_valid, error = _validate_enum_field(
            "http-ip-header",
            payload["http-ip-header"],
            VALID_BODY_HTTP_IP_HEADER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "outlook-web-access" in payload:
        is_valid, error = _validate_enum_field(
            "outlook-web-access",
            payload["outlook-web-access"],
            VALID_BODY_OUTLOOK_WEB_ACCESS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "weblogic-server" in payload:
        is_valid, error = _validate_enum_field(
            "weblogic-server",
            payload["weblogic-server"],
            VALID_BODY_WEBLOGIC_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "websphere-server" in payload:
        is_valid, error = _validate_enum_field(
            "websphere-server",
            payload["websphere-server"],
            VALID_BODY_WEBSPHERE_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-mode" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-mode",
            payload["ssl-mode"],
            VALID_BODY_SSL_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-dh-bits" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-dh-bits",
            payload["ssl-dh-bits"],
            VALID_BODY_SSL_DH_BITS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-algorithm" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-algorithm",
            payload["ssl-algorithm"],
            VALID_BODY_SSL_ALGORITHM,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-algorithm" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-algorithm",
            payload["ssl-server-algorithm"],
            VALID_BODY_SSL_SERVER_ALGORITHM,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-pfs" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-pfs",
            payload["ssl-pfs"],
            VALID_BODY_SSL_PFS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-min-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-version",
            payload["ssl-min-version"],
            VALID_BODY_SSL_MIN_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-max-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-max-version",
            payload["ssl-max-version"],
            VALID_BODY_SSL_MAX_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-min-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-min-version",
            payload["ssl-server-min-version"],
            VALID_BODY_SSL_SERVER_MIN_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-max-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-max-version",
            payload["ssl-server-max-version"],
            VALID_BODY_SSL_SERVER_MAX_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-accept-ffdhe-groups" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-accept-ffdhe-groups",
            payload["ssl-accept-ffdhe-groups"],
            VALID_BODY_SSL_ACCEPT_FFDHE_GROUPS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-send-empty-frags" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-send-empty-frags",
            payload["ssl-send-empty-frags"],
            VALID_BODY_SSL_SEND_EMPTY_FRAGS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-client-fallback" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-client-fallback",
            payload["ssl-client-fallback"],
            VALID_BODY_SSL_CLIENT_FALLBACK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-client-renegotiation" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-client-renegotiation",
            payload["ssl-client-renegotiation"],
            VALID_BODY_SSL_CLIENT_RENEGOTIATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-client-session-state-type" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-client-session-state-type",
            payload["ssl-client-session-state-type"],
            VALID_BODY_SSL_CLIENT_SESSION_STATE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-renegotiation" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-renegotiation",
            payload["ssl-server-renegotiation"],
            VALID_BODY_SSL_SERVER_RENEGOTIATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-session-state-type" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-session-state-type",
            payload["ssl-server-session-state-type"],
            VALID_BODY_SSL_SERVER_SESSION_STATE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-http-location-conversion" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-http-location-conversion",
            payload["ssl-http-location-conversion"],
            VALID_BODY_SSL_HTTP_LOCATION_CONVERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-http-match-host" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-http-match-host",
            payload["ssl-http-match-host"],
            VALID_BODY_SSL_HTTP_MATCH_HOST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hpkp" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hpkp",
            payload["ssl-hpkp"],
            VALID_BODY_SSL_HPKP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hpkp-include-subdomains" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hpkp-include-subdomains",
            payload["ssl-hpkp-include-subdomains"],
            VALID_BODY_SSL_HPKP_INCLUDE_SUBDOMAINS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hsts" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hsts",
            payload["ssl-hsts"],
            VALID_BODY_SSL_HSTS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hsts-include-subdomains" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hsts-include-subdomains",
            payload["ssl-hsts-include-subdomains"],
            VALID_BODY_SSL_HSTS_INCLUDE_SUBDOMAINS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "one-click-gslb-server" in payload:
        is_valid, error = _validate_enum_field(
            "one-click-gslb-server",
            payload["one-click-gslb-server"],
            VALID_BODY_ONE_CLICK_GSLB_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_firewall_vip_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update firewall/vip."""
    # Validate enum values using central function
    if "type" in payload:
        is_valid, error = _validate_enum_field(
            "type",
            payload["type"],
            VALID_BODY_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "server-type" in payload:
        is_valid, error = _validate_enum_field(
            "server-type",
            payload["server-type"],
            VALID_BODY_SERVER_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ldb-method" in payload:
        is_valid, error = _validate_enum_field(
            "ldb-method",
            payload["ldb-method"],
            VALID_BODY_LDB_METHOD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "src-vip-filter" in payload:
        is_valid, error = _validate_enum_field(
            "src-vip-filter",
            payload["src-vip-filter"],
            VALID_BODY_SRC_VIP_FILTER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "h2-support" in payload:
        is_valid, error = _validate_enum_field(
            "h2-support",
            payload["h2-support"],
            VALID_BODY_H2_SUPPORT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "h3-support" in payload:
        is_valid, error = _validate_enum_field(
            "h3-support",
            payload["h3-support"],
            VALID_BODY_H3_SUPPORT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "nat44" in payload:
        is_valid, error = _validate_enum_field(
            "nat44",
            payload["nat44"],
            VALID_BODY_NAT44,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "nat46" in payload:
        is_valid, error = _validate_enum_field(
            "nat46",
            payload["nat46"],
            VALID_BODY_NAT46,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "add-nat46-route" in payload:
        is_valid, error = _validate_enum_field(
            "add-nat46-route",
            payload["add-nat46-route"],
            VALID_BODY_ADD_NAT46_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "arp-reply" in payload:
        is_valid, error = _validate_enum_field(
            "arp-reply",
            payload["arp-reply"],
            VALID_BODY_ARP_REPLY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-redirect" in payload:
        is_valid, error = _validate_enum_field(
            "http-redirect",
            payload["http-redirect"],
            VALID_BODY_HTTP_REDIRECT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "persistence" in payload:
        is_valid, error = _validate_enum_field(
            "persistence",
            payload["persistence"],
            VALID_BODY_PERSISTENCE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "nat-source-vip" in payload:
        is_valid, error = _validate_enum_field(
            "nat-source-vip",
            payload["nat-source-vip"],
            VALID_BODY_NAT_SOURCE_VIP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "portforward" in payload:
        is_valid, error = _validate_enum_field(
            "portforward",
            payload["portforward"],
            VALID_BODY_PORTFORWARD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "protocol" in payload:
        is_valid, error = _validate_enum_field(
            "protocol",
            payload["protocol"],
            VALID_BODY_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "portmapping-type" in payload:
        is_valid, error = _validate_enum_field(
            "portmapping-type",
            payload["portmapping-type"],
            VALID_BODY_PORTMAPPING_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "empty-cert-action" in payload:
        is_valid, error = _validate_enum_field(
            "empty-cert-action",
            payload["empty-cert-action"],
            VALID_BODY_EMPTY_CERT_ACTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "user-agent-detect" in payload:
        is_valid, error = _validate_enum_field(
            "user-agent-detect",
            payload["user-agent-detect"],
            VALID_BODY_USER_AGENT_DETECT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "client-cert" in payload:
        is_valid, error = _validate_enum_field(
            "client-cert",
            payload["client-cert"],
            VALID_BODY_CLIENT_CERT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-cookie-domain-from-host" in payload:
        is_valid, error = _validate_enum_field(
            "http-cookie-domain-from-host",
            payload["http-cookie-domain-from-host"],
            VALID_BODY_HTTP_COOKIE_DOMAIN_FROM_HOST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-cookie-share" in payload:
        is_valid, error = _validate_enum_field(
            "http-cookie-share",
            payload["http-cookie-share"],
            VALID_BODY_HTTP_COOKIE_SHARE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "https-cookie-secure" in payload:
        is_valid, error = _validate_enum_field(
            "https-cookie-secure",
            payload["https-cookie-secure"],
            VALID_BODY_HTTPS_COOKIE_SECURE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-multiplex" in payload:
        is_valid, error = _validate_enum_field(
            "http-multiplex",
            payload["http-multiplex"],
            VALID_BODY_HTTP_MULTIPLEX,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-ip-header" in payload:
        is_valid, error = _validate_enum_field(
            "http-ip-header",
            payload["http-ip-header"],
            VALID_BODY_HTTP_IP_HEADER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "outlook-web-access" in payload:
        is_valid, error = _validate_enum_field(
            "outlook-web-access",
            payload["outlook-web-access"],
            VALID_BODY_OUTLOOK_WEB_ACCESS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "weblogic-server" in payload:
        is_valid, error = _validate_enum_field(
            "weblogic-server",
            payload["weblogic-server"],
            VALID_BODY_WEBLOGIC_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "websphere-server" in payload:
        is_valid, error = _validate_enum_field(
            "websphere-server",
            payload["websphere-server"],
            VALID_BODY_WEBSPHERE_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-mode" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-mode",
            payload["ssl-mode"],
            VALID_BODY_SSL_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-dh-bits" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-dh-bits",
            payload["ssl-dh-bits"],
            VALID_BODY_SSL_DH_BITS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-algorithm" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-algorithm",
            payload["ssl-algorithm"],
            VALID_BODY_SSL_ALGORITHM,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-algorithm" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-algorithm",
            payload["ssl-server-algorithm"],
            VALID_BODY_SSL_SERVER_ALGORITHM,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-pfs" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-pfs",
            payload["ssl-pfs"],
            VALID_BODY_SSL_PFS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-min-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-version",
            payload["ssl-min-version"],
            VALID_BODY_SSL_MIN_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-max-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-max-version",
            payload["ssl-max-version"],
            VALID_BODY_SSL_MAX_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-min-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-min-version",
            payload["ssl-server-min-version"],
            VALID_BODY_SSL_SERVER_MIN_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-max-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-max-version",
            payload["ssl-server-max-version"],
            VALID_BODY_SSL_SERVER_MAX_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-accept-ffdhe-groups" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-accept-ffdhe-groups",
            payload["ssl-accept-ffdhe-groups"],
            VALID_BODY_SSL_ACCEPT_FFDHE_GROUPS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-send-empty-frags" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-send-empty-frags",
            payload["ssl-send-empty-frags"],
            VALID_BODY_SSL_SEND_EMPTY_FRAGS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-client-fallback" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-client-fallback",
            payload["ssl-client-fallback"],
            VALID_BODY_SSL_CLIENT_FALLBACK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-client-renegotiation" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-client-renegotiation",
            payload["ssl-client-renegotiation"],
            VALID_BODY_SSL_CLIENT_RENEGOTIATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-client-session-state-type" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-client-session-state-type",
            payload["ssl-client-session-state-type"],
            VALID_BODY_SSL_CLIENT_SESSION_STATE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-renegotiation" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-renegotiation",
            payload["ssl-server-renegotiation"],
            VALID_BODY_SSL_SERVER_RENEGOTIATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-server-session-state-type" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-server-session-state-type",
            payload["ssl-server-session-state-type"],
            VALID_BODY_SSL_SERVER_SESSION_STATE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-http-location-conversion" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-http-location-conversion",
            payload["ssl-http-location-conversion"],
            VALID_BODY_SSL_HTTP_LOCATION_CONVERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-http-match-host" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-http-match-host",
            payload["ssl-http-match-host"],
            VALID_BODY_SSL_HTTP_MATCH_HOST,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hpkp" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hpkp",
            payload["ssl-hpkp"],
            VALID_BODY_SSL_HPKP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hpkp-include-subdomains" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hpkp-include-subdomains",
            payload["ssl-hpkp-include-subdomains"],
            VALID_BODY_SSL_HPKP_INCLUDE_SUBDOMAINS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hsts" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hsts",
            payload["ssl-hsts"],
            VALID_BODY_SSL_HSTS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-hsts-include-subdomains" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-hsts-include-subdomains",
            payload["ssl-hsts-include-subdomains"],
            VALID_BODY_SSL_HSTS_INCLUDE_SUBDOMAINS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "one-click-gslb-server" in payload:
        is_valid, error = _validate_enum_field(
            "one-click-gslb-server",
            payload["one-click-gslb-server"],
            VALID_BODY_ONE_CLICK_GSLB_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "firewall/vip",
    "category": "cmdb",
    "api_path": "firewall/vip",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure virtual IP for IPv4.",
    "total_fields": 98,
    "required_fields_count": 7,
    "fields_with_defaults_count": 84,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_FEATURE_SET: Literal["flow", "proxy"]
VALID_BODY_OPTIONS: Literal["activexfilter", "cookiefilter", "javafilter", "block-invalid-url", "jscript", "js", "vbs", "unknown", "intrinsic", "wf-referer", "wf-cookie", "per-user-bal"]
VALID_BODY_HTTPS_REPLACEMSG: Literal["enable", "disable"]
VALID_BODY_WEB_FLOW_LOG_ENCODING: Literal["utf-8", "punycode"]
VALID_BODY_OVRD_PERM: Literal["bannedword-override", "urlfilter-override", "fortiguard-wf-override", "contenttype-check-override"]
VALID_BODY_POST_ACTION: Literal["normal", "block"]
VALID_BODY_WISP: Literal["enable", "disable"]
VALID_BODY_WISP_ALGORITHM: Literal["primary-secondary", "round-robin", "auto-learning"]
VALID_BODY_LOG_ALL_URL: Literal["enable", "disable"]
VALID_BODY_WEB_CONTENT_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_ACTIVEX_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_COMMAND_BLOCK_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_COOKIE_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_APPLET_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_JSCRIPT_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_JS_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_VBS_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_UNKNOWN_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_REFERER_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FILTER_COOKIE_REMOVAL_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_URL_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_INVALID_DOMAIN_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FTGD_ERR_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_FTGD_QUOTA_USAGE: Literal["enable", "disable"]
VALID_BODY_EXTENDED_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_EXTENDED_ALL_ACTION_LOG: Literal["enable", "disable"]
VALID_BODY_WEB_ANTIPHISHING_LOG: Literal["enable", "disable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_FEATURE_SET",
    "VALID_BODY_OPTIONS",
    "VALID_BODY_HTTPS_REPLACEMSG",
    "VALID_BODY_WEB_FLOW_LOG_ENCODING",
    "VALID_BODY_OVRD_PERM",
    "VALID_BODY_POST_ACTION",
    "VALID_BODY_WISP",
    "VALID_BODY_WISP_ALGORITHM",
    "VALID_BODY_LOG_ALL_URL",
    "VALID_BODY_WEB_CONTENT_LOG",
    "VALID_BODY_WEB_FILTER_ACTIVEX_LOG",
    "VALID_BODY_WEB_FILTER_COMMAND_BLOCK_LOG",
    "VALID_BODY_WEB_FILTER_COOKIE_LOG",
    "VALID_BODY_WEB_FILTER_APPLET_LOG",
    "VALID_BODY_WEB_FILTER_JSCRIPT_LOG",
    "VALID_BODY_WEB_FILTER_JS_LOG",
    "VALID_BODY_WEB_FILTER_VBS_LOG",
    "VALID_BODY_WEB_FILTER_UNKNOWN_LOG",
    "VALID_BODY_WEB_FILTER_REFERER_LOG",
    "VALID_BODY_WEB_FILTER_COOKIE_REMOVAL_LOG",
    "VALID_BODY_WEB_URL_LOG",
    "VALID_BODY_WEB_INVALID_DOMAIN_LOG",
    "VALID_BODY_WEB_FTGD_ERR_LOG",
    "VALID_BODY_WEB_FTGD_QUOTA_USAGE",
    "VALID_BODY_EXTENDED_LOG",
    "VALID_BODY_WEB_EXTENDED_ALL_ACTION_LOG",
    "VALID_BODY_WEB_ANTIPHISHING_LOG",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""FortiOS CMDB - Ssl category"""

from .setting import Setting

__all__ = [
    "Setting",
    "Ssl",
]


class Ssl:
    """Ssl endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ssl endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.setting = Setting(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: log/threat_weight
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ThreatWeightWebItem:
    """Nested item for web field - supports attribute access."""
    id: int
    category: int
    level: Literal["disable", "low", "medium", "high", "critical"]


class ThreatWeightGeolocationItem:
    """Nested item for geolocation field - supports attribute access."""
    id: int
    country: str
    level: Literal["disable", "low", "medium", "high", "critical"]


class ThreatWeightApplicationItem:
    """Nested item for application field - supports attribute access."""
    id: int
    category: int
    level: Literal["disable", "low", "medium", "high", "critical"]


class ThreatWeightPayload(TypedDict, total=False):
    """Payload type for ThreatWeight operations."""
    status: Literal["enable", "disable"]
    level: str
    blocked_connection: Literal["disable", "low", "medium", "high", "critical"]
    failed_connection: Literal["disable", "low", "medium", "high", "critical"]
    url_block_detected: Literal["disable", "low", "medium", "high", "critical"]
    botnet_connection_detected: Literal["disable", "low", "medium", "high", "critical"]
    malware: str
    ips: str
    web: str | list[str] | list[dict[str, Any]] | list[ThreatWeightWebItem]
    geolocation: str | list[str] | list[dict[str, Any]] | list[ThreatWeightGeolocationItem]
    application: str | list[str] | list[dict[str, Any]] | list[ThreatWeightApplicationItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ThreatWeightResponse(TypedDict, total=False):
    """Response type for ThreatWeight - use with .dict property for typed dict access."""
    status: Literal["enable", "disable"]
    level: str
    blocked_connection: Literal["disable", "low", "medium", "high", "critical"]
    failed_connection: Literal["disable", "low", "medium", "high", "critical"]
    url_block_detected: Literal["disable", "low", "medium", "high", "critical"]
    botnet_connection_detected: Literal["disable", "low", "medium", "high", "critical"]
    malware: str
    ips: str
    web: list[ThreatWeightWebItem]
    geolocation: list[ThreatWeightGeolocationItem]
    application: list[ThreatWeightApplicationItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ThreatWeightObject(FortiObject):
    """Typed FortiObject for ThreatWeight with field access."""
    status: Literal["enable", "disable"]
    level: str
    blocked_connection: Literal["disable", "low", "medium", "high", "critical"]
    failed_connection: Literal["disable", "low", "medium", "high", "critical"]
    url_block_detected: Literal["disable", "low", "medium", "high", "critical"]
    botnet_connection_detected: Literal["disable", "low", "medium", "high", "critical"]
    malware: str
    ips: str
    web: list[ThreatWeightWebItem]
    geolocation: list[ThreatWeightGeolocationItem]
    application: list[ThreatWeightApplicationItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class ThreatWeight:
    """
    
    Endpoint: log/threat_weight
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ThreatWeightObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ThreatWeightPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        level: str | None = ...,
        blocked_connection: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        failed_connection: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        url_block_detected: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        botnet_connection_detected: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        malware: str | None = ...,
        ips: str | None = ...,
        web: str | list[str] | list[dict[str, Any]] | list[ThreatWeightWebItem] | None = ...,
        geolocation: str | list[str] | list[dict[str, Any]] | list[ThreatWeightGeolocationItem] | None = ...,
        application: str | list[str] | list[dict[str, Any]] | list[ThreatWeightApplicationItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ThreatWeightObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ThreatWeightPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        level: str | None = ...,
        blocked_connection: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        failed_connection: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        url_block_detected: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        botnet_connection_detected: Literal["disable", "low", "medium", "high", "critical"] | None = ...,
        malware: str | None = ...,
        ips: str | None = ...,
        web: str | list[str] | list[dict[str, Any]] | list[ThreatWeightWebItem] | None = ...,
        geolocation: str | list[str] | list[dict[str, Any]] | list[ThreatWeightGeolocationItem] | None = ...,
        application: str | list[str] | list[dict[str, Any]] | list[ThreatWeightApplicationItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "ThreatWeight",
    "ThreatWeightPayload",
    "ThreatWeightResponse",
    "ThreatWeightObject",
]


"""Type stubs for AUTO_CONFIG category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .custom import Custom
    from .default import Default
    from .policy import Policy

__all__ = [
    "Custom",
    "Default",
    "Policy",
    "AutoConfig",
]


class AutoConfig:
    """AUTO_CONFIG API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    custom: Custom
    default: Default
    policy: Policy

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize auto_config category with HTTP client."""
        ...
